@@ -3,9 +3,10 @@ use auto_diff::optim::{SGD, MiniBatch};
 use auto_diff::Var;
 use rand::prelude::*;
 use ::rand::prelude::StdRng;
+extern crate openblas_src;
 
 
-//use tensorboard_rs::summary_writer::SummaryWriter;
+use tensorboard_rs::summary_writer::SummaryWriter;
 
 mod mnist;
 use mnist::{load_images, load_labels};
@@ -34,6 +35,11 @@ fn main() {
     test_data.reset_net();
     test_label.reset_net();
 
+    assert_eq!(train_data.size(), [60000, 784]);
+    assert_eq!(train_label.size(), [60000]);
+    assert_eq!(test_data.size(), [10000, 784]);
+    assert_eq!(test_label.size(), [10000]);
+
 
     // build the model
 //    let mut m = Module::new();
@@ -74,45 +80,36 @@ fn main() {
     op2.set_bias(Var::normal(&mut rng, &[10, ], 0., 1.));
 
 //    //println!("{}, {}", &train_data, &train_label);
-    let mut rng = StdRng::seed_from_u64(671);
+    let rng = StdRng::seed_from_u64(671);
     let mut minibatch = MiniBatch::new(rng, 16);
 
-    //    let mut writer = SummaryWriter::new(&("./logdir".to_string()));
+    let mut writer = SummaryWriter::new(&("./logdir".to_string()));
     let (input, label) = minibatch.next(&train_data, &train_label).unwrap();        
 
     let output1 = op1.call(&[&input]).unwrap().pop().unwrap();
     let output2 = output1.sigmoid().unwrap();
     let output = op2.call(&[&output2]).unwrap().pop().unwrap();
 
-
     let loss = output.cross_entropy_loss(&label).unwrap();
     
-    let mut lr = 0.1;
+    let lr = 0.1;
     let mut opt = SGD::new(lr);    
 
-//    let mut writer = SummaryWriter::new(&("./logdir".to_string()));
-//    
-//    
+    
     for i in 0..900 {
         println!("index: {}", i);
-        //let (mdata, mlabel) = minibatch.next(&train_data, &train_label).unwrap();
         let (input_next, label_next) = minibatch.next(&train_data, &train_label).unwrap();        
         input.set(&input_next);
         label.set(&label_next);
         println!("load data done");
-        
-//        m.forward();
-//        println!("forward done");
-//        m.backward(-1.);
-//        println!("backward done");
-//        opt.step(&m);
-        //        println!("update done");
 
+        //println!("dump net: {:?}", loss.dump_net().borrow());
         loss.rerun().unwrap();
         loss.bp().unwrap();
         loss.step(&mut opt).unwrap();
 
         println!("loss: {:?}", loss);
+	writer.add_scalar(&"mlp_mnist/train_loss".to_string(), f64::try_from(loss.clone()).unwrap() as f32, i);
 
 
         if i % 10 == 0 {
