@@ -3,13 +3,18 @@ use std::collections::{BTreeSet, BTreeMap};
 use std::fmt;
 
 use crate::collection::generational_index::{GenIndex, GenKey};
-use crate::collection::graph::Graph;
+use crate::collection::directed_graph::Graph;
 use tensor_rs::tensor::Tensor;
 use crate::op::Op;
 use crate::err::AutoDiffError;
 
+#[cfg(feature = "use-serde")]
+use serde::{Serialize, Deserialize};
+
 /// The computation network.
 /// Connection has duplication.
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
+#[derive(Clone)]
 pub struct Net {
     data: GenIndex<Tensor>,
     ops: GenIndex<Op>,
@@ -55,9 +60,8 @@ impl Net {
             Err(v) => {Err(v)}
         }
     }
-    pub fn set_tensor(&mut self, id: GenKey, val: Tensor) -> Result<(), ()> {
-        self.data.replace(&id, val)?;
-        Ok(())
+    pub fn set_tensor(&mut self, id: GenKey, val: Tensor) -> Result<(), AutoDiffError> {
+        self.data.replace(&id, val)
     }
 
     /// Insert operator into the network.
@@ -350,3 +354,11 @@ impl Default for Net {
         Self::new()
     }
 }
+
+//impl PartialEq for Net {
+//    fn eq(&self, other: &Self) -> bool {
+//	unimplemented!();
+//    }
+//}
+//
+//impl Eq for Net {}
