@@ -12,7 +12,14 @@ use super::macros::{many_to_1_op_with_paras,
                     new_element_op,
                     one_to_1_op_with_paras};
 
+#[cfg(feature = "use-serde")]
+use serde::{Serialize, Deserialize};
+#[cfg(feature = "use-serde")]
+use std::any::Any;
+
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Cat {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     dim: usize
 }
@@ -45,8 +52,8 @@ impl OpCall for Cat {
 }
 impl OpTrait for Cat {
 
-    fn get_name(&self) -> String {
-        "cat".to_string()
+    fn get_name(&self) -> &'static str {
+        "Cat"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -56,8 +63,8 @@ impl OpTrait for Cat {
     }
     fn apply(&self, input: &[Tensor], output: &[Tensor]) {
         let mut new_input = vec![];
-        for i in 1..input.len() {
-            new_input.push(input[i].ref_copy());
+        for item in input.iter().skip(1) {
+            new_input.push(item.ref_copy());
         }
         output[0].swap(&input[0].cat(&new_input, self.dim));
     }
@@ -79,11 +86,15 @@ impl OpTrait for Cat {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 
 one_to_vec_op_with_paras!(Chunk,
-                          "chunk",
+                          "Chunk",
                           1,
 			  1, // TODO, this is dependent on the number of output.
 			  chunk,
@@ -95,7 +106,9 @@ one_to_vec_op_with_paras!(Chunk,
                           chunks: usize, dim: usize);
                           
 // gather
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Gather {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     dim: usize
 }
@@ -127,8 +140,8 @@ impl OpCall for Gather {
 }
 impl OpTrait for Gather {
 
-    fn get_name(&self) -> String {
-        "gather".to_string()
+    fn get_name(&self) -> &'static str {
+        "Gather"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -150,10 +163,16 @@ impl OpTrait for Gather {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 // index_select
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct IndexSelect {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     dim: usize
 }
@@ -185,8 +204,8 @@ impl OpCall for IndexSelect {
 }
 impl OpTrait for IndexSelect {
 
-    fn get_name(&self) -> String {
-        "index_select".to_string()
+    fn get_name(&self) -> &'static str {
+        "Index_select"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -208,10 +227,16 @@ impl OpTrait for IndexSelect {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 // index_exclude
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct IndexExclude {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     dim: usize
 }
@@ -243,8 +268,8 @@ impl OpCall for IndexExclude {
 }
 impl OpTrait for IndexExclude {
 
-    fn get_name(&self) -> String {
-        "index_exclude".to_string()
+    fn get_name(&self) -> &'static str {
+        "Index_exclude"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -266,10 +291,16 @@ impl OpTrait for IndexExclude {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 // reshape
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Reshape {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     new_shape: Vec<usize>,
 }
@@ -301,8 +332,8 @@ impl OpCall for Reshape {
 }
 impl OpTrait for Reshape {
 
-    fn get_name(&self) -> String {
-        "reshape".to_string()
+    fn get_name(&self) -> &'static str {
+        "Reshape"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -324,11 +355,17 @@ impl OpTrait for Reshape {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 
 // split
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Split {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     sections: Vec<usize>,
     dim: usize,
@@ -363,8 +400,8 @@ impl OpCall for Split {
 }
 impl OpTrait for Split {
 
-    fn get_name(&self) -> String {
-        "split".to_string()
+    fn get_name(&self) -> &'static str {
+        "Split"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -389,11 +426,15 @@ impl OpTrait for Split {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 // squeeze
 one_to_1_op_with_paras!(Squeeze,
-                        "squeeze",
+                        "Squeeze",
                         1, 1,
                         squeeze,
                         (|input: &[Tensor],
@@ -406,7 +447,7 @@ one_to_1_op_with_paras!(Squeeze,
 
 // stack
 many_to_1_op_with_paras!(Stack,
-                          "stack",
+                          "Stack",
                           2, // TODO, this is dependent on the number of input.
                           1,
                           stack,
@@ -418,7 +459,7 @@ many_to_1_op_with_paras!(Stack,
                           dim: usize);
 // t
 new_element_op!(T,
-                "t",
+                "T",
                 t,
                 (|input: &[Tensor],
                  output_grad: &[Tensor],
@@ -427,7 +468,9 @@ new_element_op!(T,
                  }));
 
 // take
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Take {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     sizes: Vec<usize>,
 }
@@ -459,8 +502,8 @@ impl OpCall for Take {
 }
 impl OpTrait for Take {
 
-    fn get_name(&self) -> String {
-        "take".to_string()
+    fn get_name(&self) -> &'static str {
+        "Take"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -482,10 +525,16 @@ impl OpTrait for Take {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 // permute
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Permute {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     sizes: Vec<usize>,
 }
@@ -517,8 +566,8 @@ impl OpCall for Permute {
 }
 impl OpTrait for Permute {
 
-    fn get_name(&self) -> String {
-        "permute".to_string()
+    fn get_name(&self) -> &'static str {
+        "Permute"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -540,12 +589,16 @@ impl OpTrait for Permute {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 
 
 // unsqueeze
 one_to_1_op_with_paras!(Unsqueeze,
-                        "unsqueeze",
+                        "Unsqueeze",
                         1, 1,
                         unsqueeze,
                         (|input: &[Tensor],
@@ -556,7 +609,9 @@ one_to_1_op_with_paras!(Unsqueeze,
                         dim: usize);
 
 // conditional_select
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct ConditionalSelect {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
 }
 impl ConditionalSelect {
@@ -585,8 +640,8 @@ impl OpCall for ConditionalSelect {
 }
 impl OpTrait for ConditionalSelect {
 
-    fn get_name(&self) -> String {
-        "conditional_select".to_string()
+    fn get_name(&self) -> &'static str {
+        "ConditionalSelect"
     }
     fn get_input_size(&self) -> usize {
         3
@@ -608,6 +663,10 @@ impl OpTrait for ConditionalSelect {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 impl Default for ConditionalSelect {
     fn default() -> Self {
@@ -617,7 +676,9 @@ impl Default for ConditionalSelect {
 
 
 // repeat
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Repeat {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
     sizes: Vec<usize>,
 }
@@ -649,8 +710,8 @@ impl OpCall for Repeat {
 }
 impl OpTrait for Repeat {
 
-    fn get_name(&self) -> String {
-        "repeat".to_string()
+    fn get_name(&self) -> &'static str {
+        "Repeat"
     }
     fn get_input_size(&self) -> usize {
         1
@@ -672,4 +733,8 @@ impl OpTrait for Repeat {
     }
     fn set_values(&self, _v: &[Tensor]) {
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
