@@ -7,7 +7,12 @@ use std::rc::Rc;
 use crate::var::{Var};
 use crate::err::AutoDiffError;
 
+#[cfg(feature = "use-serde")]
+use serde::{Serialize, Deserialize};
+#[cfg(feature = "use-serde")]
+use std::any::Any;
 
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct Linear {
     in_fea: Option<usize>,
     out_fea: Option<usize>,
@@ -16,7 +21,7 @@ pub struct Linear {
     bias: Tensor,
     weight_grad: Tensor,
     bias_grad: Tensor,
-    
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
 }
 impl Linear {
@@ -79,8 +84,8 @@ impl OpTrait for Linear {
 
     
 
-    fn get_name(&self) -> String {
-        "Linear".to_string()
+    fn get_name(&self) -> &'static str {
+        "Linear"
     }
 
     fn get_input_size(&self) -> usize {
@@ -151,10 +156,16 @@ impl OpTrait for Linear {
         }
         ret
     }
+
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
     
 }
 
 // Bilinear
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct BiLinear {
     in1_fea: Option<usize>,
     in2_fea: Option<usize>,
@@ -164,7 +175,7 @@ pub struct BiLinear {
     bias: Tensor,
     weight_grad: Tensor,
     bias_grad: Tensor,
-    
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
 }
 impl BiLinear {
@@ -227,8 +238,8 @@ impl OpCall for BiLinear {
 }
 
 impl OpTrait for BiLinear {
-    fn get_name(&self) -> String {
-        "BiLinear".to_string()
+    fn get_name(&self) -> &'static str {
+        "BiLinear"
     }
 
     fn get_input_size(&self) -> usize {
@@ -285,5 +296,10 @@ impl OpTrait for BiLinear {
         }
         ret
     }
+
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
     
 }
