@@ -1,6 +1,10 @@
 use tensor_rs::tensor::Tensor;
 use super::{OpTrait, OpHandle};
 
+#[cfg(feature = "use-serde")]
+use serde::{Serialize, Deserialize};
+#[cfg(feature = "use-serde")]
+use std::any::Any;
 
 //
 // Common Cost function
@@ -15,7 +19,9 @@ pub enum Reduction{
 
 /// MSELoss
 /// The left-most dimension is the N.
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct MSELoss {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
  }
 impl MSELoss {
@@ -29,8 +35,8 @@ impl MSELoss {
 impl OpTrait for MSELoss {
 
     
-    fn get_name(&self) -> String {
-        "MSE".to_string()
+    fn get_name(&self) -> &'static str {
+        "MSE"
     }
     fn get_input_size(&self) -> usize {
         2
@@ -81,6 +87,10 @@ impl OpTrait for MSELoss {
     fn get_grads(&self) -> Vec<Tensor> {
         Vec::new()
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 impl Default for MSELoss {
     fn default() -> Self {
@@ -90,7 +100,9 @@ impl Default for MSELoss {
 
 
 /// CrossEntropyLoss
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct CrossEntropyLoss {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
 }
 impl CrossEntropyLoss {
@@ -102,8 +114,8 @@ impl CrossEntropyLoss {
     handle_method!();
 }
 impl OpTrait for CrossEntropyLoss {
-    fn get_name(&self) -> String {
-        "CrossEntropyLoss".to_string()
+    fn get_name(&self) -> &'static str {
+        "CrossEntropyLoss"
     }
     fn get_input_size(&self) -> usize {
         2
@@ -191,6 +203,10 @@ impl OpTrait for CrossEntropyLoss {
     fn get_grads(&self) -> Vec<Tensor> {
         Vec::new()
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 impl Default for CrossEntropyLoss {
     fn default() -> Self {
@@ -211,7 +227,9 @@ impl Default for CrossEntropyLoss {
 /// -y log (1/(1 + exp(-x))) - (1-y) log(1 - 1/(1 + exp(-x)))
 /// 
 /// Prediction comes first, label comes second.
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct BCEWithLogitsLoss {
+    #[cfg_attr(feature = "use-serde", serde(skip))]
     handle: OpHandle,
 }
 impl BCEWithLogitsLoss {
@@ -224,8 +242,8 @@ impl BCEWithLogitsLoss {
 }
 impl OpTrait for BCEWithLogitsLoss {
     
-    fn get_name(&self) -> String {
-        "BCEWithLogitsLoss".to_string()
+    fn get_name(&self) -> &'static str {
+        "BCEWithLogitsLoss"
     }
     fn get_input_size(&self) -> usize {
         2
@@ -278,6 +296,10 @@ impl OpTrait for BCEWithLogitsLoss {
     fn get_grads(&self) -> Vec<Tensor> {
         Vec::new()
     }
+    #[cfg(feature = "use-serde")]
+    fn as_any(&self) -> &dyn Any {
+	self
+    }
 }
 impl Default for BCEWithLogitsLoss {
     fn default() -> Self {
