@@ -187,6 +187,15 @@ impl VarInner {
         }
     }
 
+    pub fn get_id(&self) -> GenKey {
+	self.id
+    }
+    pub fn get_need_grad(&self) -> bool {
+	self.need_grad
+    }
+    pub fn get_net(&self) -> Rc<RefCell<Net>> {
+	self.net.clone()
+    }
 
     pub fn size(&self) -> Vec<usize> {
         self.net.borrow().get_tensor(self.id).expect("").size()
@@ -204,20 +213,22 @@ impl VarInner {
 	}
     }
     pub fn get_f32(&self, o: &[usize]) -> Result<f32, AutoDiffError> {
-	Self::check_index(&self, o)?;
+	Self::check_index(self, o)?;
         Ok(self.net.borrow().get_tensor(self.id)?.get_f32(o))
     }
     pub fn set_f32(&mut self, o: &[usize], v: f32) -> Result<(), AutoDiffError> {
-	Self::check_index(&self, o)?;
-        Ok(self.net.borrow().get_tensor(self.id)?.set_f32(o, v))
+	Self::check_index(self, o)?;
+	self.net.borrow().get_tensor(self.id)?.set_f32(o, v);
+        Ok(())
     }
     pub fn get_f64(&self, o: &[usize]) -> Result<f64, AutoDiffError> {
-	Self::check_index(&self, o)?;
+	Self::check_index(self, o)?;
         Ok(self.net.borrow().get_tensor(self.id)?.get_f64(o))
     }
     pub fn set_f64(&mut self, o: &[usize], v: f64) -> Result<(), AutoDiffError>{
-	Self::check_index(&self, o)?;
-        Ok(self.net.borrow().get_tensor(self.id)?.set_f64(o, v))
+	Self::check_index(self, o)?;
+	self.net.borrow().get_tensor(self.id)?.set_f64(o, v);
+        Ok(())
     }
 
     pub fn fill(size: &[usize], fill_value: Rc<RefCell<VarInner>>) -> VarInner {
@@ -331,6 +342,13 @@ impl VarInner {
         Ok(())
     }
 
+    pub fn get_io_var(&self) -> Result<(Vec<VarInner>, Vec<VarInner>), AutoDiffError> {
+	let input_id = self.net.borrow().get_input_edge_data();
+	let output_id = self.net.borrow().get_output_edge_data();
+	Ok((input_id.iter().map(|x| VarInner {id: *x, need_grad: true, net: self.net.clone()}).collect(),
+	    output_id.iter().map(|x| VarInner {id: *x, need_grad: true, net: self.net.clone()}).collect(),))
+    }
+
     pub(crate) fn set_grad(&mut self, use_gradient: bool) {
         self.need_grad = use_gradient;
     }
@@ -367,7 +385,7 @@ impl VarInner {
             for set in other_var_by_networks {
                 let mut old_ids = vec![];
                 for item in &set {
-                    old_ids.push(item.borrow().id.clone());
+                    old_ids.push(item.borrow().id);
                 }
                 let other_key = self.net.borrow_mut().append(
                     &set[0].borrow().net.borrow(), &old_ids)?;
@@ -601,6 +619,14 @@ impl VarInner {
     pub fn dump_net(&self) -> Rc<RefCell<Net>> {
         self.net.clone()
     }
+
+    pub(crate) fn set_inner(id: GenKey, need_grad: bool, net: Net) -> VarInner {
+	VarInner {
+	    id,
+	    need_grad,
+	    net: Rc::new(RefCell::new(net))
+	}
+    }
 }
 
 impl PartialEq for VarInner {
